/*
 * Copyright © 2007 Global Locate.
 *
 * Written by Ken Yale.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#include <gtk/gtk.h>
#include <gdk/gdk.h>
#include <stdlib.h>
#include <string.h>
#include "agps.h"
#include "sv_bar.h"

// TODO:
// 1)  Connect the needle to real GPS activities
// 2)  Add numbers and tick marks to the dials.

void notebook_expose(const char* pageName);

static GtkDrawingArea* speed;
static GdkPixmap*      speed_PIX = 0;
static PangoLayout*    speed_text;

static GdkGC* blue_gc = 0;
static GdkGC* grey_gc = 0;

extern GdkGC* alloc_color(GdkPixmap* pm, const char* color);

static void
speed_alloc_colors()
{
    if (!speed_PIX) return;
    if (!blue_gc) blue_gc = alloc_color(speed_PIX, "Blue");
    if (!grey_gc) grey_gc = alloc_color(speed_PIX, "SlateGrey");
}

static void
speed_clear()
{
    if (!speed_PIX) return;
    GtkWidget* w = GTK_WIDGET(speed);
    gdk_draw_rectangle(speed_PIX, w->style->white_gc, TRUE,
                       0, 0,
                       w->allocation.width,
                       w->allocation.height);
}

static gboolean
configure_speed_event(GtkWidget* w, GdkEventConfigure* event)
{
    if (speed_PIX)
    {
        g_object_unref(speed_PIX);
    }

    //--------------------------------------------------------------------------
    //          pixmap
    //--------------------------------------------------------------------------
    speed_PIX = gdk_pixmap_new(w->window,
                            w->allocation.width,
                            w->allocation.height, -1);
    speed_clear();
    return TRUE;
}

static gboolean
expose_speed_event(GtkWidget* w, GdkEventExpose* event)
{
    notebook_expose("SP");
    if (speed_PIX)
    {
        GdkRectangle& a = event->area;
        gdk_draw_drawable(w->window,
                      w->style->fg_gc[GTK_WIDGET_STATE(w)],
                      speed_PIX, a.x, a.y, a.x, a.y, a.width, a.height);
    }
    return FALSE;
}

static GdkGC*
getGC(bool used)
{
    speed_alloc_colors();
    GdkGC*    gc = grey_gc;
    if (used) gc = blue_gc;
    if (!gc)  gc = GTK_WIDGET(speed)->style->black_gc;
    return gc;
}

enum
{
    CENTER_XY = 3,
    CENTER_X  = 1,
    CENTER_Y  = 2,
    CENTER_NONE = 0,

    ALIGN_BOTTOM = 16,
    ALIGN_RIGHT  = 32
};

static void
speed_draw_text(int x, int y, const char* t,
                int center = CENTER_XY, GdkGC* gc = 0)
{
    if (!gc) gc = getGC(false);
    pango_layout_set_text(speed_text, t, -1);
    if (center)
    {
        int w;
        int h;
        pango_layout_get_pixel_size(speed_text, &w, &h);
        if      (center & CENTER_X    ) x -= w/2;
        else if (center & ALIGN_RIGHT ) x -= w;
        if      (center & CENTER_Y    ) y -= h/2;
        else if (center & ALIGN_BOTTOM) y -= h;
    }
    // printf("speed_draw_text(%d,%d,\"%s\", %d)\n", x, y, t, center);
    gdk_draw_layout(speed_PIX, gc, x, y, speed_text);
}

static void
speed_draw_line(int x0, int y0, int x1, int y1)
{
    // printf("speed_draw_line(%d,%d,%d,%d)\n", x0, y0, x1, y1);
    GdkGC* gc = GTK_WIDGET(speed)->style->black_gc;
    gdk_draw_line(speed_PIX, gc, x0, y0, x1, y1);
}

static void
speed_draw_dial(const char* name, int x, int y)
{
    GdkGC* gc = getGC(true);

    int radius = PIXMAP_WIDTH/4 - 3;
    int width  = radius * 2;
    int height = width;
    int arc0 = 0;
    int arcN = 360 * DEGREE_64;
    gdk_draw_arc(speed_PIX, gc, FALSE,
                x+3, y+3,
                width, height,
                arc0, arcN);

    int cx = x + PIXMAP_WIDTH/4;
    int cy = y + PIXMAP_WIDTH/4;
    speed_draw_text(cx, cy-radius/2, name);

    speed_draw_line(cx,cy,  cx,cy-radius);
}

static void
speed_draw_dials()
{
    int h = PIXMAP_WIDTH/2;

    speed_draw_dial("SPEED",   0, 0);
    speed_draw_dial("ALT",     h, 0);
    speed_draw_dial("HEADING", 0, h);
    speed_draw_dial("VERTICAL\nSPEED", h, h);

    gtk_widget_queue_draw_area(GTK_WIDGET(speed), 0, 0,
                               PIXMAP_WIDTH, PIXMAP_WIDTH);
}

void
page_speed_done()
{
    if (!speed_PIX) return;
    speed_clear();
    speed_draw_dials();
}

GtkWidget*
page_speed_create(GtkWidget* window)
{
    GtkWidget* label;
    GtkWidget* w;
    GtkWidget* table;

    //--------------------------------------------------------------------------
    //          table
    //--------------------------------------------------------------------------
    table = gtk_table_new(14, 2, TRUE);

    //--------------------------------------------------------------------------
    //          label
    //--------------------------------------------------------------------------
    label = gtk_label_new("Indicators");
    gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
    gtk_table_attach_defaults(GTK_TABLE (table), label, 0, 2, 0, 1);
    gtk_widget_show(label);

    speed_text = gtk_widget_create_pango_layout(label, "");

    //--------------------------------------------------------------------------
    //          window for the various dials
    //--------------------------------------------------------------------------
    w  = gtk_drawing_area_new();
    speed = GTK_DRAWING_AREA(w);
    gtk_drawing_area_size(speed, PIXMAP_WIDTH, PIXMAP_WIDTH);

    gtk_signal_connect(GTK_OBJECT(w), "configure_event",
                    G_CALLBACK(configure_speed_event), w);

    gtk_signal_connect(GTK_OBJECT(w), "expose_event",
                    G_CALLBACK(expose_speed_event), w);

    gtk_widget_set_events(w, GDK_EXPOSURE_MASK);
    gtk_table_attach_defaults(GTK_TABLE (table), w, 0, 2, 1, 14);
    gtk_widget_show(w);

    gtk_widget_show(table);
    return table;
}

void
page_speed_report(FILE* f)
{
    fprintf(f, "Say something about %s/%d\n", __FILE__, __LINE__);
}
